from typing import Union

def remove_trailing_string(content: str, trailing: str) -> str: ...
def dedent(content: Union[str, bytes]) -> str: ...
def camelcase_to_spaces(content: str) -> str: ...
def markup_description(description: str) -> str: ...



from typing import Any, NoReturn

def strict_constant(o: Any) -> NoReturn: ...
def dump(*args: Any, **kwargs: Any) -> None: ...
def dumps(*args: Any, **kwargs: Any) -> str: ...
def load(*args: Any, **kwargs: Any) -> Any: ...
def loads(*args: Any, **kwargs: Any) -> Any: ...



from typing import Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Tuple, Type, Union

from django.core.paginator import Paginator, SupportsCount, SupportsLen
from django.db.models import QuerySet
from django.http import HttpRequest
from django.views import View
from rest_framework.request import Request

def _positive_int(integer_string: str, strict: bool = ..., cutoff: Optional[int] = ...) -> int: ...
def _divide_with_ceil(a: int, b: int) -> int: ...
def _get_displayed_page_numbers(current: int, final: int) -> List[Optional[int]]: ...
def _get_page_links(
    page_numbers: Sequence[Optional[int]], current: int, url_func: Callable[[int], str]
) -> List[PageLink]: ...
def _reverse_ordering(ordering_tuple: Sequence[str]) -> Tuple[str, ...]: ...

class Cursor(NamedTuple):
    offset: int
    reverse: bool
    position: Optional[str]

class PageLink(NamedTuple):
    url: Optional[str]
    number: Optional[int]
    is_active: bool
    is_break: bool

PAGE_BREAK: PageLink = ...

class BasePagination(object):
    display_page_controls: bool = ...
    def paginate_queryset(self, queryset: QuerySet, request: HttpRequest, view: Optional[View] = ...): ...
    def get_paginated_response(self, data: Any) -> Optional[List[Any]]: ...
    def to_html(self) -> str: ...
    def get_results(self, data: Dict[str, Any]) -> Any: ...
    def get_schema_fields(self, view: View) -> List[Any]: ...

class PageNumberPagination(BasePagination):
    page_size: int = ...

    django_paginator_class: Type[Paginator] = ...
    page_query_param: str = ...
    page_query_description: str = ...

    page_size_query_param: Optional[str] = ...
    page_size_query_description: str = ...
    max_page_size: Optional[int] = ...

    last_page_strings: Sequence[str] = ...

    template: str = ...

    invalid_page_message: str = ...
    def get_page_size(self, request: Request) -> int: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_html_context(self) -> Dict[str, Any]: ...

class LimitOffsetPagination(BasePagination):
    default_limit: int = ...
    limit_query_param: str = ...
    limit_query_description: str = ...
    offset_query_param: str = ...
    offset_query_description: str = ...
    max_limit: Optional[int] = ...
    template: str = ...
    def get_limit(self, request: Request) -> int: ...
    def get_offset(self, request: Request) -> int: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_html_context(self) -> Dict[str, Any]: ...
    def get_count(self, queryset: Union[SupportsCount, SupportsLen]) -> int: ...

class CursorPagination(BasePagination):
    cursor_query_param: str = ...
    cursor_query_description: str = ...
    page_size: int = ...
    invalid_cursor_message: str = ...
    ordering: str = ...
    template: str = ...
    page_size_query_param: Optional[str] = ...
    page_size_query_description: str = ...
    max_page_size: Optional[int] = ...
    offset_cutoff: int = ...
    def get_page_size(self, request: Request) -> Optional[int]: ...
    def get_next_link(self) -> Optional[str]: ...
    def get_previous_link(self) -> Optional[str]: ...
    def get_ordering(self, request: Request, queryset: QuerySet, view: View) -> Tuple[str, ...]: ...
    def decode_cursor(self, request: Request) -> Optional[Cursor]: ...
    def encode_cursor(self, cursor: Cursor) -> str: ...
    def _get_position_from_instance(self, instance: Any, ordering: Sequence[str]) -> str: ...
    def get_html_context(self) -> Dict[str, str]: ...



from contextlib import contextmanager
from types import TracebackType
from typing import Any, ContextManager, Dict, Iterator, Optional, Sequence, Tuple, Type

from django.db.models.base import Model
from django.http import HttpRequest, QueryDict
from rest_framework.authentication import BaseAuthentication
from rest_framework.negotiation import BaseContentNegotiation
from rest_framework.parsers import BaseParser
from rest_framework.views import APIView

def is_form_media_type(media_type: str) -> bool: ...

class override_method(ContextManager["Request"]):
    def __init__(self, view: APIView, request: Request, method: str): ...
    def __enter__(self) -> Request: ...
    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]: ...

class WrappedAttributeError(Exception): ...

@contextmanager
def wrap_attributeerrors() -> Iterator[None]: ...

class Empty: ...

def _hasattr(obj: Any, name: str) -> bool: ...
def clone_request(request: Request, method: str) -> Request: ...

class ForcedAuthentication:
    def __init__(self, force_user: Optional[Model], force_token: Optional[Any]) -> None: ...
    def authenticate(self, request: Request) -> Tuple[Optional[Model], Optional[Any]]: ...

class Request(HttpRequest):
    user: Any
    auth: Any

    parsers: Optional[Sequence[BaseParser]] = ...
    authenticators: Optional[Sequence[BaseAuthentication]] = ...
    negotiator: Optional[BaseContentNegotiation] = ...
    parser_context: Optional[Dict[str, Any]] = ...
    def __init__(
        self,
        request: HttpRequest,
        parsers: Optional[Sequence[BaseParser]] = ...,
        authenticators: Optional[Sequence[BaseAuthentication]] = ...,
        negotiator: Optional[BaseContentNegotiation] = ...,
        parser_context: Optional[Dict[str, Any]] = ...,
    ) -> None: ...
    def _default_negotiator(self) -> BaseContentNegotiation: ...
    @property
    def content_type(self) -> str: ...  # type: ignore
    @property
    def stream(self) -> Any: ...
    @property
    def query_params(self) -> QueryDict: ...
    @property
    def data(self) -> Dict[str, Any]: ...
    @property
    def successful_authenticator(self) -> Optional[BaseAuthentication]: ...
    def force_plaintext_errors(self, value: Any) -> None: ...
    def __getattr__(self, attr: str) -> Any: ...



from typing import Callable, Dict, Optional, Sequence

from mypy.nodes import TypeInfo
from mypy.options import Options
from mypy.plugin import AttributeContext, ClassDefContext, FunctionContext, MethodContext, Plugin
from mypy.types import Instance, Type

from mypy_drf_plugin import helpers, monkeypatch
from mypy_drf_plugin.transformers import fields, serializers, validation


def extract_base_model_fullname(serializer_type: Type) -> Optional[str]:
    if not isinstance(serializer_type, Instance):
        return None

    base_model_fullname = helpers.get_drf_metadata(serializer_type.type).get('base_model')
    if not base_model_fullname:
        return None

    return base_model_fullname


def get_instance_of_model_bound_to_serializer(ctx: MethodContext) -> Type:
    base_model_fullname = extract_base_model_fullname(ctx.type)
    if not base_model_fullname:
        return ctx.default_return_type

    try:
        return ctx.api.named_generic_type(base_model_fullname, [])
    except AssertionError:  # cannot find a model with name
        return ctx.default_return_type


class BaseClassTransformers:
    def __init__(self, hooks: Sequence[Callable]):
        self.hooks = hooks

    def __call__(self, ctx: ClassDefContext) -> None:
        for hook in self.hooks:
            hook(ctx)


class DRFPlugin(Plugin):
    def __init__(self, options: Options) -> None:
        super().__init__(options)
        monkeypatch.make_field_repr_not_return_any_generics()

    def _get_currently_defined_serializers(self) -> Dict[str, int]:
        base_serializer_sym = self.lookup_fully_qualified(helpers.BASE_SERIALIZER_FULLNAME)
        if base_serializer_sym is not None and isinstance(base_serializer_sym.node, TypeInfo):
            return (base_serializer_sym.node.metadata
                    .setdefault('drf', {})
                    .setdefault('serializer_bases', {helpers.BASE_SERIALIZER_FULLNAME: 1,
                                                     helpers.MODEL_SERIALIZER_FULLNAME: 1,
                                                     helpers.SERIALIZER_FULLNAME: 1}))
        else:
            return {}

    def _get_currently_defined_list_serializers(self) -> Dict[str, int]:
        list_serializer_sym = self.lookup_fully_qualified(helpers.LIST_SERIALIZER_FULLNAME)
        if list_serializer_sym is not None and isinstance(list_serializer_sym.node, TypeInfo):
            return (helpers.get_drf_metadata(list_serializer_sym.node)
                    .setdefault('list_serializer_bases', {helpers.LIST_SERIALIZER_FULLNAME: 1}))
        else:
            return {}

    def _get_defined_serializer_base_classes(self) -> Dict[str, int]:
        return {**self._get_currently_defined_serializers(), **self._get_currently_defined_list_serializers()}

    def _get_defined_field_base_classes(self) -> Dict[str, int]:
        base_field_sym = self.lookup_fully_qualified(helpers.FIELD_FULLNAME)
        if base_field_sym is not None and isinstance(base_field_sym.node, TypeInfo):
            return (helpers.get_drf_metadata(base_field_sym.node)
                    .setdefault('field_bases', {helpers.FIELD_FULLNAME: 1}))
        else:
            return {}

    def get_function_hook(self, fullname: str
                          ) -> Optional[Callable[[FunctionContext], Type]]:
        sym = self.lookup_fully_qualified(fullname)
        if sym is not None and isinstance(sym.node, TypeInfo):
            if sym.node.has_base(helpers.FIELD_FULLNAME):
                return fields.set_generic_parameters_for_field

    def get_method_hook(self, fullname: str
                        ) -> Optional[Callable[[MethodContext], Type]]:
        class_name, _, method_name = fullname.rpartition('.')
        if method_name == 'to_representation':
            if class_name in self._get_currently_defined_serializers():
                return validation.return_typeddict_from_to_representation

            if class_name in self._get_currently_defined_list_serializers():
                return validation.return_list_of_typeddict_for_list_serializer_from_to_representation

        if method_name in {'to_internal_value', 'run_validation'}:
            if class_name in self._get_currently_defined_serializers():
                return validation.return_typeddict_from_to_internal_value

            if class_name in self._get_currently_defined_list_serializers():
                return validation.return_list_of_typeddict_for_list_serializer_from_to_internal_value

        if method_name in {'create', 'save'}:
            if class_name in self._get_currently_defined_serializers():
                return get_instance_of_model_bound_to_serializer

    def get_base_class_hook(self, fullname: str
                            ) -> Optional[Callable[[ClassDefContext], None]]:
        if fullname in self._get_defined_serializer_base_classes():
            return BaseClassTransformers([serializers.transform_serializer_class,
                                          fields.set_types_metadata])

        if fullname in self._get_defined_field_base_classes():
            return fields.set_types_metadata


def plugin(version):
    return DRFPlugin



from typing import cast

from mypy.nodes import StrExpr, TypeInfo, TupleExpr, ListExpr, SetExpr
from mypy.plugin import ClassDefContext
from mypy.semanal import SemanticAnalyzerPass2

from mypy_drf_plugin import helpers


def make_meta_nested_class_inherit_from_any(ctx: ClassDefContext) -> None:
    meta_node = helpers.get_nested_meta_node_for_current_class(ctx.cls.info)
    if meta_node is None:
        return None
    meta_node.fallback_to_any = True


def save_model_class_to_serializer_metadata(ctx: ClassDefContext) -> None:
    base_model = helpers.get_meta_attribute_value(ctx, 'model')
    if not base_model:
        return None

    api = cast(SemanticAnalyzerPass2, ctx.api)
    if isinstance(base_model, StrExpr):
        base_model_fullname = helpers.get_model_fullname_from_string(base_model.value,
                                                                     all_modules=api.modules)
    else:
        base_model_fullname = base_model.fullname

    helpers.get_drf_metadata(ctx.cls.info)['base_model'] = base_model_fullname


def save_fields_sequence_to_serializer_metadata(ctx: ClassDefContext) -> None:
    fields = helpers.get_meta_attribute_value(ctx, 'fields')
    if not fields:
        return None

    field_names = []
    if isinstance(fields, (TupleExpr, ListExpr, SetExpr)):
        for field_expr in fields.items:
            if isinstance(field_expr, StrExpr):
                field_names.append(field_expr.value)

    helpers.get_drf_metadata(ctx.cls.info)['fields'] = field_names


def transform_serializer_class(ctx: ClassDefContext) -> None:
    if ctx.cls.info.has_base(helpers.LIST_SERIALIZER_FULLNAME):
        sym = ctx.api.lookup_fully_qualified_or_none(helpers.LIST_SERIALIZER_FULLNAME)
        if sym is not None and isinstance(sym.node, TypeInfo):
            helpers.get_drf_metadata(sym.node)['list_serializer_bases'][ctx.cls.fullname] = 1
    else:
        sym = ctx.api.lookup_fully_qualified_or_none(helpers.BASE_SERIALIZER_FULLNAME)
        if sym is not None and isinstance(sym.node, TypeInfo):
            helpers.get_drf_metadata(sym.node)['serializer_bases'][ctx.cls.fullname] = 1

        save_model_class_to_serializer_metadata(ctx)
        save_fields_sequence_to_serializer_metadata(ctx)

    make_meta_nested_class_inherit_from_any(ctx)



from typing import Any, Callable, Dict, List, NoReturn, Optional, Sequence, Tuple, Type

from django.http import HttpRequest
from django.views.generic import View
from rest_framework.authentication import BaseAuthentication
from rest_framework.negotiation import BaseContentNegotiation
from rest_framework.parsers import BaseParser
from rest_framework.permissions import BasePermission
from rest_framework.renderers import BaseRenderer
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.schemas.inspectors import ViewInspector
from rest_framework.settings import APISettings
from rest_framework.throttling import BaseThrottle
from rest_framework.versioning import BaseVersioning

def get_view_name(view: APIView) -> str: ...
def get_view_description(view: APIView, html: bool = ...) -> str: ...
def set_rollback() -> None: ...
def exception_handler(exc: Exception, context) -> Optional[Response]: ...

class APIView(View):
    renderer_classes: Sequence[Type[BaseRenderer]] = ...
    parser_classes: Sequence[Type[BaseParser]] = ...
    authentication_classes: Sequence[Type[BaseAuthentication]] = ...
    throttle_classes: Sequence[Type[BaseThrottle]] = ...
    permission_classes: Sequence[Type[BasePermission]] = ...
    content_negotiation_class: Optional[str] = ...
    metadata_class: Optional[str] = ...
    versioning_class: Optional[str] = ...
    settings: APISettings
    schema: ViewInspector = ...
    @classmethod
    def as_view(cls, **initkwargs: Any) -> Any: ...
    @property
    def allowed_methods(self) -> List[str]: ...
    @property
    def default_response_headers(self) -> Dict[str, str]: ...
    def permission_denied(self, request: Request, message: Optional[str] = ...) -> NoReturn: ...
    def throttled(self, request: Request, wait: float) -> NoReturn: ...
    def get_authenticate_header(self, request: Request) -> str: ...
    def get_parser_context(self, http_request: HttpRequest) -> Dict[str, Any]: ...
    def get_renderer_context(self) -> Dict[str, Any]: ...
    def get_exception_handler_context(self) -> Dict[str, Any]: ...
    def get_view_name(self) -> str: ...
    def get_view_description(self, html: bool = ...) -> str: ...
    def get_format_suffix(self, **kwargs: Any) -> Optional[str]: ...
    def get_renderers(self) -> List[BaseRenderer]: ...
    def get_parsers(self) -> List[BaseParser]: ...
    def get_authenticators(self) -> List[BaseAuthentication]: ...
    def get_permissions(self) -> List[BasePermission]: ...
    def get_throttles(self) -> List[BaseThrottle]: ...
    def get_content_negotiator(self) -> BaseContentNegotiation: ...
    def get_exception_handler(self) -> Callable: ...
    def perform_content_negotiation(self, request: Request, force: bool = ...) -> Tuple[BaseRenderer, str]: ...
    def perform_authentication(self, request: Request) -> None: ...
    def check_permissions(self, request: Request) -> None: ...
    def check_object_permissions(self, request: Request, obj: Any) -> None: ...
    def check_throttles(self, request: Request) -> None: ...
    def determine_version(
        self, request: Request, *args: Any, **kwargs: Any
    ) -> Tuple[Optional[str], Optional[BaseVersioning]]: ...
    def initialize_request(self, request: Request, *args: Any, **kwargs: Any) -> Request: ...
    def initial(self, request: Request, *args: Any, **kwargs: Any) -> None: ...
    def finalize_response(self, request: Request, response: Response, *args: Any, **kwargs: Any) -> Response: ...
    def handle_exception(self, exc: Exception) -> Response: ...
    def raise_uncaught_exception(self, exc: Exception) -> NoReturn: ...



from typing import Any, Dict, List, Optional, Sequence, Type, TypeVar, Union, Iterable

from django.db.models import Manager, Model
from django.db.models.query import QuerySet
from rest_framework.filters import BaseFilterBackend
from rest_framework.pagination import BasePagination
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer

from rest_framework import mixins, views

_T = TypeVar("_T", bound=Model)

def get_object_or_404(
    queryset: Union[Type[_T], Manager[_T], QuerySet[_T]], *filter_args: Any, **filter_kwargs: Any
) -> _T: ...

class GenericAPIView(views.APIView):
    queryset: Optional[QuerySet] = ...
    serializer_class: Optional[Type[BaseSerializer]] = ...
    lookup_field: str = ...
    lookup_url_kwarg: Optional[str] = ...

    filter_backends: Sequence[Type[BaseFilterBackend]] = ...
    pagination_class: Optional[Type[BasePagination]] = ...
    def get_queryset(self) -> QuerySet: ...
    def get_object(self) -> Any: ...
    def get_serializer(self, *args: Any, **kwargs: Any) -> BaseSerializer: ...
    def get_serializer_class(self) -> Type[BaseSerializer]: ...
    def get_serializer_context(self) -> Dict[str, Any]: ...
    def filter_queryset(self, queryset: QuerySet) -> QuerySet: ...
    @property
    def paginator(self) -> Optional[BasePagination]: ...
    def paginate_queryset(self, queryset: QuerySet) -> List[Any]: ...
    def get_paginated_response(self, data: Any) -> Response: ...

class CreateAPIView(mixins.CreateModelMixin, GenericAPIView):
    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class ListAPIView(mixins.ListModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveAPIView(mixins.RetrieveModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class DestroyAPIView(mixins.DestroyModelMixin, GenericAPIView):
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class UpdateAPIView(mixins.UpdateModelMixin, GenericAPIView):
    def put(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class ListCreateAPIView(mixins.ListModelMixin, mixins.CreateModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveUpdateAPIView(mixins.RetrieveModelMixin, mixins.UpdateModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def put(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveDestroyAPIView(mixins.RetrieveModelMixin, mixins.DestroyModelMixin, GenericAPIView):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveUpdateDestroyAPIView(
    mixins.RetrieveModelMixin, mixins.UpdateModelMixin, mixins.DestroyModelMixin, GenericAPIView
):
    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def put(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def patch(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def delete(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...



from typing import Any, Dict, IO, Mapping, Optional, Type

from rest_framework.renderers import BaseRenderer

class DataAndFiles(object):
    data: Any
    files: Mapping[str, IO[Any]]
    def __init__(self, data: Any, files: Mapping[str, IO[Any]]) -> None: ...

class BaseParser(object):
    media_type: str = ...
    def parse(
        self, stream: IO[Any], media_type: Optional[str] = ..., parser_context: Optional[Mapping[str, Any]] = ...
    ) -> Dict[str, Any]: ...

class JSONParser(BaseParser):
    renderer_class: Type[BaseRenderer] = ...
    strict: bool = ...

class FormParser(BaseParser): ...
class MultiPartParser(BaseParser): ...

class FileUploadParser(BaseParser):
    errors: Dict[str, str] = ...
    def get_filename(self, stream: IO[Any], media_type: Optional[str], parser_context: Mapping[str, Any]) -> str: ...
    def get_encoded_filename(self, filename_parm: Mapping[str, Any]) -> str: ...



from json import JSONEncoder
from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Type

from django import forms
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer
from rest_framework.utils.field_mapping import ClassLookupDict
from rest_framework.views import APIView

def zero_as_none(value: Any) -> Optional[Any]: ...

class BaseRenderer(object):
    media_type: str = ...
    format: str = ...
    charset: Optional[str] = ...
    render_style: str = ...
    def render(
        self, data: Any, accepted_media_type: Optional[str] = ..., renderer_context: Optional[Mapping[str, Any]] = ...
    ) -> Any: ...

class JSONRenderer(BaseRenderer):
    encoder_class: Type[JSONEncoder] = ...
    ensure_ascii: bool = ...
    compact: bool = ...
    strict: bool = ...
    def get_indent(self, accepted_media_type: str, renderer_context: Mapping[str, Any]) -> Optional[int]: ...

class TemplateHTMLRenderer(BaseRenderer):
    template_name: Optional[str] = ...
    exception_template_names: Sequence[str] = ...
    def resolve_template(self, template_names: Iterable[str]) -> Any: ...
    def get_template_context(self, data: Any, renderer_context: Mapping[str, Any]): ...
    def get_template_names(self, response: Response, view: APIView) -> List[str]: ...
    def get_exception_template(self, response: Response) -> Any: ...

class StaticHTMLRenderer(TemplateHTMLRenderer): ...

class HTMLFormRenderer(BaseRenderer):
    template_pack: str = ...
    base_template: str = ...

    default_style: ClassLookupDict = ...
    def render_field(self, field, parent_style: Mapping[str, Any]) -> str: ...

class BrowsableAPIRenderer(BaseRenderer):
    """
    HTML renderer used to self-document the API.
    """

    template: str = ...
    filter_template: str = ...
    code_style: str = ...
    form_renderer_class: Type[BaseRenderer] = ...
    def get_default_renderer(self, view: APIView) -> BaseRenderer: ...
    def get_content(
        self, renderer: BaseRenderer, data: Any, accepted_media_type: str, renderer_context: Mapping[str, Any]
    ) -> str: ...
    def show_form_for_method(self, view: APIView, method: str, request: Request, obj: Any) -> bool: ...
    def _get_serializer(
        self, serializer_class: Type[BaseSerializer], view_instance: APIView, request: Request, *args, **kwargs
    ) -> BaseSerializer: ...
    def get_rendered_html_form(self, data: Any, view: APIView, method: str, request: Request) -> Any: ...
    def render_form_for_serializer(self, serializer: BaseSerializer) -> Any: ...
    def get_raw_data_form(self, data: Any, view: APIView, method: str, request: Request) -> Optional[forms.Form]: ...
    def get_name(self, view: APIView) -> str: ...
    def get_description(self, view: APIView, status_code: int) -> str: ...
    def get_breadcrumbs(self, request: Request) -> List[Tuple[str, str]]: ...
    def get_extra_actions(self, view: APIView) -> Optional[Dict[str, str]]: ...
    def get_filter_form(self, data: Any, view: APIView, request: Request) -> Optional[Any]: ...
    def get_context(
        self, data: Any, accepted_media_type: str, renderer_context: Mapping[str, Any]
    ) -> Dict[str, Any]: ...

class AdminRenderer(BrowsableAPIRenderer):
    def get_result_url(self, result: Mapping[str, Any], view: APIView) -> Optional[str]: ...

class DocumentationRenderer(BaseRenderer):
    template: str = ...
    error_template: str = ...
    code_style: str = ...
    languages: Sequence[str] = ...
    def get_context(self, data: Any, request: Request) -> Dict[str, Any]: ...

class SchemaJSRenderer(BaseRenderer):
    template: str = ...

class MultiPartRenderer(BaseRenderer):
    BOUNDARY: str = ...

class CoreJSONRenderer(BaseRenderer): ...

class _BaseOpenAPIRenderer:
    def get_schema(self, instance: Any) -> Dict[str, Any]: ...
    def get_parameters(self, link) -> Dict[str, Any]: ...
    def get_operation(self, link, name, tag) -> Dict[str, Any]: ...
    def get_paths(self, document) -> Dict[str, Any]: ...
    def get_structure(self, data: Any) -> Dict[str, Any]: ...

class OpenAPIRenderer(_BaseOpenAPIRenderer): ...
class JSONOpenAPIRenderer(_BaseOpenAPIRenderer): ...



import datetime
import decimal
import uuid
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    List,
    NoReturn,
    Optional,
    Pattern,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    Iterable,
    Protocol,
    Type,
    Mapping,
    MutableMapping,
)

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Model
from django.forms import FilePathField as DjangoFilePathField, ImageField as DjangoImageField
from rest_framework.serializers import BaseSerializer

from rest_framework.relations import Option

class empty: ...

def is_simple_callable(obj: Callable) -> bool: ...

class CreateOnlyDefault(object):
    def __init__(self, default: Any) -> None: ...
    def set_context(self, serializer_field: Field) -> None: ...
    def __call__(self) -> Any: ...

class SkipField(Exception): ...

def set_value(dictionary: MutableMapping[str, Any], keys: Sequence[str], value: Any) -> None: ...
def get_error_detail(exc_info: ValidationError) -> Any: ...

_FT = TypeVar("_FT")  # Field Type
_FPT = TypeVar("_FPT")  # Field Primitive Type

_Validator = Callable[[Any], None]

class Field(Generic[_FT, _FPT]):
    _pyi_field_actual_type: Any
    _pyi_field_primitive_type: Any

    default_error_messages: Dict[str, str] = ...
    default_validators: List[Callable] = ...
    default_empty_html: Any = ...
    initial: Optional[Any] = ...
    # TODO: add Generic/Plugin support for parent
    parent: Optional[Any]
    validators: Optional[List[_Validator]]
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def bind(self, field_name: str, parent: BaseSerializer) -> None: ...
    def get_validators(self) -> List[Callable]: ...
    def get_initial(self) -> Any: ...
    def get_value(self, dictionary: Any) -> Any: ...
    def get_attribute(self, instance: Model) -> Optional[Any]: ...
    def get_default(self) -> Any: ...
    def validate_empty_values(self, data: Any) -> Tuple[bool, Any]: ...
    def run_validation(self, data: Any = ...) -> Any: ...
    def run_validators(self, value: Any) -> None: ...
    def to_internal_value(self, data: Any) -> Any: ...
    def to_representation(self, value: Any) -> Any: ...
    def fail(self, key: str, **kwargs: Any) -> NoReturn: ...
    @property
    def root(self) -> BaseSerializer: ...
    @property
    def context(self) -> Dict[str, Any]: ...

class BooleanField(Field[_FT, _FPT]):
    _pyi_field_actual_type: bool
    _pyi_field_primitive_type: bool

    TRUE_VALUES: Set[Any] = ...
    FALSE_VALUES: Set[Any] = ...
    NULL_VALUES: Set[Optional[Any]] = ...

class NullBooleanField(Field[_FT, _FPT]):
    _pyi_field_actual_type: Optional[bool]
    _pyi_field_primitive_type: Optional[bool]

    TRUE_VALUES: Set[Any] = ...
    FALSE_VALUES: Set[Any] = ...
    NULL_VALUES: Set[Optional[Any]] = ...

class CharField(Field[_FT, _FPT]):
    _pyi_field_actual_type: str
    _pyi_field_primitive_type: str
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        min_length: int = ...,
        max_length: int = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class EmailField(CharField[_FT, _FPT]): ...

class RegexField(CharField[_FT, _FPT]):
    def __init__(
        self,
        regex: Union[str, Pattern],
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        min_length: int = ...,
        max_length: int = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class SlugField(CharField[_FT, _FPT]):
    def __init__(
        self,
        allow_unicode: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        min_length: int = ...,
        max_length: int = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class URLField(CharField[_FT, _FPT]): ...

class UUIDField(Field[_FT, _FPT]):
    _pyi_field_actual_type: uuid.UUID
    _pyi_field_primitive_type: str

    valid_formats: Sequence[str] = ...
    def __init__(
        self,
        format: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class IPAddressField(CharField[_FT, _FPT]):
    def __init__(
        self,
        protocol: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        min_length: int = ...,
        max_length: int = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class IntegerField(Field[_FT, _FPT]):
    _pyi_field_actual_type: int
    _pyi_field_primitive_type: int

    MAX_STRING_LENGTH: str = ...
    re_decimal: Pattern = ...
    def __init__(
        self,
        min_value: int = ...,
        max_value: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class FloatField(Field[_FT, _FPT]):
    _pyi_field_actual_type: float
    _pyi_field_primitive_type: float

    MAX_STRING_LENGTH: int = ...
    def __init__(
        self,
        min_value: float = ...,
        max_value: float = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class DecimalField(Field[_FT, _FPT]):
    MAX_STRING_LENGTH: int = ...
    def __init__(
        self,
        max_digits: Optional[int],
        decimal_places: Optional[int],
        coerce_to_string: Optional[bool] = ...,
        max_value: Optional[Any] = ...,
        min_value: Optional[Any] = ...,
        localize: bool = ...,
        rounding: Optional[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def validate_precision(self, value: decimal.Decimal) -> decimal.Decimal: ...
    def quantize(self, value: decimal.Decimal) -> decimal.Decimal: ...

class DateTimeField(Field[_FT, _FPT]):
    _pyi_field_actual_type: datetime.datetime
    _pyi_field_primitive_type: str

    datetime_parser: Callable = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Optional[Sequence[str]] = ...,
        default_timezone: Optional[Union[str, datetime.tzinfo]] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def enforce_timezone(self, value: Any) -> Any: ...
    def default_timezone(self) -> Optional[str]: ...

class DateField(Field[_FT, _FPT]):
    _pyi_field_actual_type: datetime.date
    _pyi_field_primitive_type: str

    datetime_parser: Callable = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Optional[Sequence[str]] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class TimeField(Field[_FT, _FPT]):
    _pyi_field_actual_type: datetime.time
    _pyi_field_primitive_type: str

    datetime_parser: Callable = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Optional[Sequence[str]] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class DurationField(Field[_FT, _FPT]):
    max_value: Any
    min_value: Any
    def __init__(
        self,
        max_value: object = ...,
        min_value: object = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

# Choice types...

class ChoiceField(Field[_FT, _FPT]):
    choices: Sequence[Any]
    html_cutoff: Optional[int] = ...
    html_cutoff_text: Optional[str] = ...
    def __init__(
        self,
        choices: Sequence[Any],
        html_cutoff: Optional[int] = ...,
        html_cutoff_text: Optional[str] = ...,
        allow_blank: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def iter_options(self) -> Iterable[Option]: ...
    def _get_choices(self) -> Sequence[Any]: ...
    def _set_choices(self, choices: Sequence[Any]) -> None: ...

class MultipleChoiceField(ChoiceField[_FT, _FPT]):
    def __init__(
        self,
        choices: Sequence[Any],
        html_cutoff: Optional[int] = ...,
        html_cutoff_text: Optional[str] = ...,
        allow_blank: bool = ...,
        allow_empty: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class FilePathField(ChoiceField[_FT, _FPT]):
    def __init__(
        self,
        path: str = ...,
        match: Optional[str] = ...,
        choices: Sequence[Any] = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: Optional[bool] = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class FileField(Field[_FT, _FPT]):
    max_length: int
    allow_empty_file: bool
    def __init__(
        self,
        allow_empty_file: bool = ...,
        max_length: int = ...,
        use_url: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class SupportsToPython(Protocol):
    def to_python(self, value: Any) -> Any: ...

class ImageField(FileField[_FT, _FPT]):
    _DjangoImageField: SupportsToPython
    def __init__(
        self,
        _DjangoImageField: Type[SupportsToPython] = ...,
        allow_empty: bool = ...,
        max_length: int = ...,
        min_length: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class _UnvalidatedField(Field[_FT, _FPT]): ...

class ListField(Field[_FT, _FPT]):
    child: Field = ...
    allow_empty: bool = ...
    max_length: int = ...
    min_length: int = ...
    def __init__(
        self,
        child: Field = ...,
        allow_empty: bool = ...,
        max_length: int = ...,
        min_length: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def run_child_validation(self, data: Any) -> Any: ...

class DictField(Field[_FT, _FPT]):
    child: Field = ...
    def __init__(
        self,
        child: Field = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    def run_child_validation(self, data: Any) -> Any: ...

class HStoreField(DictField[_FT, _FPT]):
    child: CharField = ...

class JSONField(Field[_FT, _FPT]):
    def __init__(
        self,
        binary: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class ReadOnlyField(Field[_FT, _FPT]): ...
class HiddenField(Field[_FT, _FPT]): ...

class SerializerMethodField(Field[_FT, _FPT]):
    method_name: str
    def __init__(
        self,
        method_name: Optional[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...

class ModelField(Field[_FT, _FPT]):
    model_field: models.Field
    def __init__(
        self,
        model_field: models.Field,
        max_length: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...



from typing import Any, Callable, Optional, Sequence, Type

from rest_framework.authentication import BaseAuthentication
from rest_framework.parsers import BaseParser
from rest_framework.permissions import BasePermission
from rest_framework.renderers import BaseRenderer
from rest_framework.schemas.inspectors import ViewInspector
from rest_framework.throttling import BaseThrottle

_ViewCallable = Callable
_ViewDecorator = Callable[[Callable], _ViewCallable]

def api_view(http_method_names: Optional[Sequence[str]] = ...) -> _ViewDecorator: ...
def renderer_classes(renderer_classes: Sequence[Type[BaseRenderer]]) -> _ViewDecorator: ...
def parser_classes(parser_classes: Sequence[Type[BaseParser]]) -> _ViewDecorator: ...
def authentication_classes(authentication_classes: Sequence[Type[BaseAuthentication]]) -> _ViewDecorator: ...
def throttle_classes(throttle_classes: Sequence[Type[BaseThrottle]]) -> _ViewDecorator: ...
def permission_classes(permission_classes: Sequence[Type[BasePermission]]) -> _ViewDecorator: ...
def schema(view_inspector: Type[ViewInspector]) -> _ViewDecorator: ...
def action(
    methods: Optional[Sequence[str]] = ...,
    detail: Optional[bool] = ...,
    url_path: Optional[str] = ...,
    url_name: Optional[str] = ...,
    **kwargs: Any
) -> _ViewDecorator: ...

class MethodMapper(dict):
    def __init__(self, action: Callable, methods: Sequence[str]) -> None: ...
    def _map(self, method: str, func: Callable) -> Callable: ...
    def get(self, func: Callable) -> Callable: ...  # type: ignore
    def post(self, func: Callable) -> Callable: ...
    def put(self, func: Callable) -> Callable: ...
    def patch(self, func: Callable) -> Callable: ...
    def delete(self, func: Callable) -> Callable: ...
    def head(self, func: Callable) -> Callable: ...
    def options(self, func: Callable) -> Callable: ...
    def trace(self, func: Callable) -> Callable: ...

def detail_route(methods: Optional[Sequence[str]] = ..., **kwargs: Any) -> _ViewDecorator: ...
def list_route(methods: Optional[Sequence[str]] = ..., **kwargs: Any) -> _ViewDecorator: ...



from mypy.types import Instance, Type

from mypy_drf_plugin import helpers


def make_field_repr_not_return_any_generics():
    from mypy import messages

    # format_bare
    old_format_bare = messages.MessageBuilder.format_bare

    def patched_format_bare(self, typ: Type, verbosity: int = 0) -> str:
        if isinstance(typ, Instance) and typ.type.has_base(helpers.FIELD_FULLNAME):
            typ = helpers.reparametrize_instance(typ, [])

        return old_format_bare(self, typ, verbosity)

    messages.MessageBuilder.format_bare = patched_format_bare



import re
from typing import Any, List, Mapping, Optional, Sequence, Type

from django.db import models
from django.db.models.base import Model
from rest_framework.fields import Field

header_regex = re.compile("^[a-zA-Z][0-9A-Za-z_]*:")

def field_to_schema(field: Field) -> Any: ...
def get_pk_description(model: Type[Model], model_field: models.Field) -> str: ...

class ViewInspector(object):
    instance_schemas: Mapping[str, Any]
    view: Any
    def __init__(self) -> None: ...
    def get_link(self, path: str, method: str, base_url: str) -> Any: ...

class AutoSchema(ViewInspector):
    def __init__(self, manual_fields: Optional[Sequence[Any]] = ...) -> None: ...
    def get_description(self, path: str, method: str) -> str: ...
    def get_path_fields(self, path: str, method: str) -> List[Any]: ...
    def get_serializer_fields(self, path: str, method: str) -> List[Any]: ...
    def get_pagination_fields(self, path: str, method: str) -> List[Any]: ...
    def get_filter_fields(self, path: str, method: str) -> List[Any]: ...
    def get_manual_fields(self, path: str, method: str) -> List[Any]: ...
    @staticmethod
    def update_fields(fields: List[Any], update_with: List[Any]) -> List[Any]: ...
    def get_encoding(self, path: str, method: str) -> Optional[str]: ...

class ManualSchema(ViewInspector):
    def __init__(self, fields: Sequence[Any], description: str = ..., encoding: Optional[str] = ...) -> None: ...

class DefaultSchema(ViewInspector): ...



from typing import Any, Callable, Optional, Sequence, Type

from rest_framework.renderers import BaseRenderer

from .generators import SchemaGenerator as SchemaGenerator
from .inspectors import AutoSchema as AutoSchema, DefaultSchema as DefaultSchema, ManualSchema as ManualSchema

def get_schema_view(
    title: Optional[str] = ...,
    url: Optional[str] = ...,
    description: Optional[str] = ...,
    urlconf: Optional[str] = ...,
    renderer_classes: Optional[Sequence[Type[BaseRenderer]]] = ...,
    public: bool = ...,
    patterns: Optional[Sequence[Any]] = ...,
    generator_class: Type[SchemaGenerator] = ...,
    authentication_classes: Sequence[str] = ...,
    permission_classes: Sequence[str] = ...,
) -> Callable[..., Any]: ...



from typing import Any, Optional, Sequence, Tuple, Union, Dict

from django.db.models import Model
from django.http import HttpRequest, HttpResponse
from django.test import testcases
from django.test.client import Client as DjangoClient, ClientHandler, RequestFactory as DjangoRequestFactory
from rest_framework.compat import requests
from rest_framework.response import Response

def force_authenticate(request: HttpRequest, user: Optional[Model] = ..., token: Optional[Any] = ...) -> None: ...

class RequestsClient(requests.Session): ...

class APIRequestFactory(DjangoRequestFactory):
    renderer_classes_list: Sequence[str] = ...
    default_format: str = ...
    def __init__(self, enforce_csrf_checks: bool = ..., **defaults: Any) -> None: ...
    def _encode_data(
        self, data: Optional[Any], format: Optional[str] = ..., content_type: Optional[str] = ...
    ) -> Tuple[bytes, str]: ...

class ForceAuthClientHandler(ClientHandler):
    def __init__(self, *args: Any, **kwargs: Any): ...
    def get_response(self, request: HttpRequest) -> HttpResponse: ...

class APIClient(DjangoClient):
    def credentials(self, **kwargs: Any): ...
    def force_authenticate(self, user: Optional[Model] = ..., token: Optional[Any] = ...) -> None: ...
    def request(self, **request: Any) -> Response: ...
    def get(self, path: str, data: Any = ..., secure: bool = ..., **extra: Any) -> Response: ...
    def post(
        self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> Response: ...
    def head(self, path: str, data: Any = ..., secure: bool = ..., **extra: Any) -> Response: ...
    def trace(self, path: str, secure: bool = ..., **extra: Any) -> Response: ...
    def options(
        self,
        path: str,
        data: Union[Dict[str, str], str] = ...,
        content_type: str = ...,
        secure: bool = ...,
        **extra: Any
    ) -> Response: ...
    def put(
        self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> Response: ...
    def patch(
        self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> Response: ...
    def delete(
        self, path: str, data: Any = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> Response: ...
    def generic(
        self,
        method: str,
        path: str,
        data: Any = ...,
        content_type: Optional[str] = ...,
        secure: bool = ...,
        **extra: Any
    ) -> Response: ...

class APITransactionTestCase(testcases.TransactionTestCase): ...
class APITestCase(testcases.TestCase): ...
class APISimpleTestCase(testcases.SimpleTestCase): ...
class APILiveServerTestCase(testcases.LiveServerTestCase): ...
class URLPatternsTestCase(testcases.SimpleTestCase): ...



from typing import (
    Dict,
    Sequence,
    Optional,
    Any,
    Iterable,
    List,
    Callable,
    Mapping,
    Type,
    Tuple,
    NoReturn,
    MutableMapping,
    Iterator,
)

from rest_framework.fields import (
    Field as Field,
    CharField as CharField,
    RegexField as RegexField,
    EmailField as EmailField,
    URLField as URLField,
    FileField as FileField,
    IntegerField as IntegerField,
    FloatField as FloatField,
    BooleanField as BooleanField,
    NullBooleanField as NullBooleanField,
    ListField as ListField,
    DictField as DictField,
    ChoiceField as ChoiceField,
    JSONField as JSONField,
    DateTimeField as DateTimeField,
    SerializerMethodField as SerializerMethodField,
    HiddenField as HiddenField,
    DecimalField as DecimalField,
    HStoreField as HStoreField,
    SlugField as SlugField,
    UUIDField as UUIDField,
    FilePathField as FilePathField,
    TimeField as TimeField,
    MultipleChoiceField as MultipleChoiceField,
    DateField as DateField,
    DurationField as DurationField,
    IPAddressField as IPAddressField,
    ModelField as ModelField,
    ReadOnlyField as ReadOnlyField,
    ImageField as ImageField,
    SkipField as SkipField,
    CreateOnlyDefault as CreateOnlyDefault,
)
from rest_framework.exceptions import (
    APIException as APIException,
    ValidationError as ValidationError,
    ParseError as ParseError,
    NotAuthenticated as NotAuthenticated,
    PermissionDenied as PermissionDenied,
    ErrorDetail as ErrorDetail,
    AuthenticationFailed as AuthenticationFailed,
    MethodNotAllowed as MethodNotAllowed,
    NotAcceptable as NotAcceptable,
    NotFound as NotFound,
    Throttled as Throttled,
    UnsupportedMediaType as UnsupportedMediaType,
)
from rest_framework.relations import (
    RelatedField as RelatedField,
    PrimaryKeyRelatedField as PrimaryKeyRelatedField,
    HyperlinkedRelatedField as HyperlinkedRelatedField,
    HyperlinkedIdentityField as HyperlinkedIdentityField,
    ManyRelatedField as ManyRelatedField,
    SlugRelatedField as SlugRelatedField,
    StringRelatedField as StringRelatedField,
    Hyperlink as Hyperlink,
)
from django.core.exceptions import ValidationError as DjangoValidationError
from django.db import models
from django.db.models import DurationField as ModelDurationField, Model
from django.db.models.fields import Field as DjangoModelField
from django.utils.translation import ugettext_lazy as _
from rest_framework.utils.serializer_helpers import BoundField, ReturnDict, ReturnList
from rest_framework.utils.model_meta import FieldInfo, RelationInfo

LIST_SERIALIZER_KWARGS: Sequence[str] = ...
ALL_FIELDS: str = ...

class BaseSerializer(Field):
    _declared_fields: Dict[str, Field]
    instance: Optional[Any]
    initial_data: Any
    partial: bool
    def __init__(self, instance: Optional[Any] = ..., data: Any = ..., **kwargs: Any): ...
    @classmethod
    def many_init(cls, *args: Any, **kwargs: Any) -> BaseSerializer: ...
    def to_internal_value(self, data: Any) -> Any: ...
    def to_representation(self, instance: Any) -> Any: ...
    def update(self, instance: Model, validated_data: Any) -> Any: ...
    def create(self, validated_data: Any) -> Any: ...
    def save(self, **kwargs: Any) -> Any: ...
    def is_valid(self, raise_exception: bool = ...) -> bool: ...
    @property
    def data(self) -> Any: ...
    @property
    def errors(self) -> Iterable[Any]: ...
    @property
    def validated_data(self) -> Any: ...

class SerializerMetaclass(type):
    @classmethod
    def _get_declared_fields(cls, bases: Sequence[type], attrs: Dict[str, Any]) -> Dict[str, Field]: ...

def as_serializer_error(exc: Exception) -> Dict[str, List[ErrorDetail]]: ...

class Serializer(BaseSerializer, Mapping[str, BoundField]):
    @property
    def fields(self) -> Dict[str, Field]: ...
    @property
    def _writable_fields(self) -> List[Field]: ...
    @property
    def _readable_fields(self) -> List[Field]: ...
    def get_fields(self) -> Dict[str, Field]: ...
    def _read_only_defaults(self) -> Dict[str, Any]: ...
    def validate(self, attrs):
        return attrs
    @property
    def data(self) -> ReturnDict: ...
    @property
    def errors(self) -> ReturnDict: ...
    def __iter__(self) -> Iterator[str]: ...
    def __getitem__(self, key: str) -> BoundField: ...
    def __len__(self) -> int: ...

class ListSerializer(BaseSerializer):
    child: BaseSerializer = ...
    many: bool = ...
    def validate(self, attrs):
        return attrs
    @property
    def data(self) -> ReturnList: ...
    @property
    def errors(self) -> ReturnList: ...

def raise_errors_on_nested_writes(method_name: str, serializer: BaseSerializer, validated_data: Any) -> None: ...

class ModelSerializer(Serializer):
    serializer_field_mapping: Dict[Type[models.Field], Type[Field]] = ...
    serializer_related_field: Type[RelatedField] = ...
    serializer_related_to_field: Type[RelatedField] = ...
    serializer_url_field: Type[RelatedField] = ...
    serializer_choice_field: Type[Field] = ...
    url_field_name: Optional[str] = ...
    def get_field_names(self, declared_fields: Mapping[str, Field], info: FieldInfo) -> List[str]: ...
    def get_default_field_names(self, declared_fields: Mapping[str, Field], model_info: FieldInfo) -> List[str]: ...
    def build_field(
        self, field_name: str, info: FieldInfo, model_class: Type[Model], nested_depth: int
    ) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_standard_field(
        self, field_name: str, model_field: Type[models.Field]
    ) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_relational_field(
        self, field_name: str, relation_info: RelationInfo
    ) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_nested_field(
        self, field_name: str, relation_info: RelationInfo, nested_depth: int
    ) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_property_field(self, field_name: str, model_class: Type[Model]) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_url_field(self, field_name: str, model_class: Type[Model]) -> Tuple[Type[Field], Dict[str, Any]]: ...
    def build_unknown_field(self, field_name: str, model_class: Type[Model]) -> NoReturn: ...
    def include_extra_kwargs(
        self, kwargs: MutableMapping[str, Any], extra_kwargs: MutableMapping[str, Any]
    ) -> MutableMapping[str, Any]: ...
    def get_extra_kwargs(self) -> Dict[str, Any]: ...
    def get_uniqueness_extra_kwargs(
        self, field_names: Iterable[str], declared_fields: Mapping[str, Field], extra_kwargs: Dict[str, Any]
    ) -> Tuple[Dict[str, Any], Dict[str, HiddenField]]: ...
    def _get_model_fields(
        self, field_names: Iterable[str], declared_fields: Mapping[str, Field], extra_kwargs: MutableMapping[str, Any]
    ) -> Dict[str, models.Field]: ...
    def get_unique_together_validators(self) -> List[Callable]: ...
    def get_unique_for_date_validators(self) -> List[Callable]: ...

class HyperlinkedModelSerializer(ModelSerializer): ...



from typing import Any, Dict, Optional, Pattern, Sequence

from rest_framework.request import Request

class BaseVersioning:
    default_version: Optional[str] = ...
    allowed_versions: Optional[Sequence[str]] = ...
    version_param: str = ...
    def determine_version(self, request: Request, *args: Any, **kwargs: Any) -> str: ...
    def reverse(
        self,
        viewname: str,
        args: Optional[Any] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        request: Optional[Request] = ...,
        format: Optional[str] = ...,
        **extra: Any
    ) -> str: ...
    def is_allowed_version(self, version: Optional[str]) -> bool: ...

class AcceptHeaderVersioning(BaseVersioning):
    invalid_version_message: str = ...

class URLPathVersioning(BaseVersioning):
    invalid_version_message: str = ...

class NamespaceVersioning(BaseVersioning):
    invalid_version_message: str = ...
    def get_versioned_viewname(self, viewname: str, request: Request) -> str: ...

class HostNameVersioning(BaseVersioning):
    hostname_regex: Pattern = ...
    invalid_version_message: str = ...

class QueryParameterVersioning(BaseVersioning):
    invalid_version_message: str = ...



from collections import Counter, OrderedDict
from typing import Any, Dict, Iterable, List, Optional, Pattern, Sequence, Tuple, Type, Union

from django.db.models.base import Model
from django.http.request import HttpRequest
from rest_framework.compat import coreapi

def common_path(paths: Iterable[str]) -> str: ...
def get_pk_name(model: Type[Model]) -> str: ...
def is_api_view(callback: Any) -> bool: ...

INSERT_INTO_COLLISION_FMT: str = ...

class LinkNode(OrderedDict):
    links: List[Any]
    methods_counter: Counter
    def get_available_key(self, preferred_key: str) -> str: ...

def insert_into(target: LinkNode, keys: Sequence[str], value: Any) -> None: ...
def distribute_links(obj: LinkNode) -> None: ...
def is_custom_action(action: str) -> bool: ...

_APIEndpoint = Tuple[str, str, Any]

def endpoint_ordering(endpoint: _APIEndpoint) -> Tuple[str, int]: ...

_PATH_PARAMETER_COMPONENT_RE: Pattern = ...

class EndpointEnumerator(object):
    patterns: List[Any]
    def __init__(self, patterns: Optional[List[Any]] = ..., urlconf: Optional[str] = ...) -> None: ...
    def get_api_endpoints(self, patterns: Optional[List[Any]] = ..., prefix: str = ...) -> List[_APIEndpoint]: ...
    def get_path_from_regex(self, path_regex: Union[Pattern, str]) -> str: ...
    def should_include_endpoint(self, path: str, callback: Any) -> bool: ...
    def get_allowed_methods(self, callback: Any) -> List[str]: ...

class SchemaGenerator(object):
    urlconf: Optional[str]
    title: Optional[str]
    description: Optional[str]
    patterns: List[Any]
    url: Optional[str]
    endpoints: Optional[Sequence[_APIEndpoint]]

    default_mapping: Dict[str, str] = ...
    endpoint_inspector_cls: Type[EndpointEnumerator] = ...
    coerce_method_names: Optional[Dict[str, str]] = ...

    coerce_path_pk: Optional[bool] = ...
    def __init__(
        self,
        title: Optional[str] = ...,
        url: Optional[str] = ...,
        description: Optional[str] = ...,
        patterns: Optional[List[Any]] = ...,
        urlconf: Optional[str] = ...,
    ) -> None: ...
    def get_schema(self, request: Optional[HttpRequest] = ..., public: bool = ...) -> Optional[coreapi.Document]: ...
    def get_links(self, request: Optional[HttpRequest] = ...) -> Optional[LinkNode]: ...
    def determine_path_prefix(self, paths: Sequence[str]) -> str: ...
    def create_view(self, callback: Any, method: str, request: Optional[HttpRequest] = ...) -> Any: ...
    def has_view_permissions(self, path: str, method: str, view) -> bool: ...
    def coerce_path(self, path: str, method: str, view) -> str: ...
    def get_keys(self, subpath: str, method: str, view) -> List[str]: ...



from typing import Any, Dict, Mapping, Optional

from django.template.response import SimpleTemplateResponse

class Response(SimpleTemplateResponse):
    data: Optional[Any] = ...
    exception: bool = ...
    content_type: Optional[str] = ...
    def __init__(
        self,
        data: Optional[Any] = ...,
        status: Optional[int] = ...,
        template_name: Optional[str] = ...,
        headers: Optional[Mapping[str, Any]] = ...,
        exception: bool = ...,
        content_type: Optional[str] = ...,
    ): ...
    @property
    def status_text(self) -> str: ...



from typing import Dict, NamedTuple, Optional

from django.db.models import Model
from django.db.models.fields import Field
from django.db.models.fields.related import RelatedField

class RelationInfo(NamedTuple):
    model_field: Optional[RelatedField]
    related_model: Model
    to_many: bool
    to_field: str
    has_through_model: bool
    reverse: bool

class FieldInfo(NamedTuple):
    pk: Field
    fields: Dict[str, Field]
    forward_relations: Dict[str, RelationInfo]
    reverse_relations: Dict[str, RelationInfo]
    fields_and_pk: Dict[str, Field]
    relations: Dict[str, RelationInfo]

def get_field_info(model: Model) -> FieldInfo: ...
def is_abstract_model(model: Model) -> bool: ...



from collections import OrderedDict
from typing import Any, Callable, List

from rest_framework.request import Request

from rest_framework import generics, mixins, views

def _is_extra_action(attr: Any) -> bool: ...

class ViewSetMixin(object):
    basename: str
    request: Any
    @classmethod
    def as_view(cls, **initkwargs: Any) -> Callable: ...
    def initialize_request(self, request: Request, *args: Any, **kwargs: Any) -> Request: ...
    def reverse_action(self, url_name: str, *args: Any, **kwargs: Any) -> str: ...
    @classmethod
    def get_extra_actions(cls) -> List[Any]: ...
    def get_extra_action_url_map(self) -> OrderedDict[str, str]: ...

class ViewSet(ViewSetMixin, views.APIView): ...
class GenericViewSet(ViewSetMixin, generics.GenericAPIView): ...
class ReadOnlyModelViewSet(mixins.RetrieveModelMixin, mixins.ListModelMixin, GenericViewSet): ...
class ModelViewSet(
    mixins.CreateModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    mixins.ListModelMixin,
    GenericViewSet,
): ...



from typing import Any, Container, Iterable, MutableMapping, Optional, TypeVar

from django.db.models import Model, QuerySet
from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer

def qs_exists(queryset: QuerySet) -> bool: ...

_T = TypeVar("_T", bound=Model)

def qs_filter(queryset: QuerySet[_T], **kwargs: Any) -> QuerySet[_T]: ...

class UniqueValidator(object):
    message: str = ...

    queryset: QuerySet
    serializer_field: Optional[Field]
    lookup: str
    field_name: str
    instance: Optional[Any]
    def __init__(self, queryset: QuerySet, message: Optional[str] = ..., lookup: str = ...) -> None: ...
    def set_context(self, serializer_field: Field) -> None: ...
    def filter_queryset(self, value: Any, queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def exclude_current_instance(self, queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def __call__(self, value: Any) -> None: ...

class UniqueTogetherValidator(object):
    message: str = ...
    missing_message: str = ...

    queryset: QuerySet
    fields: Iterable[str]
    serializer_field: Optional[Field]
    instance: Optional[Model]
    def __init__(self, queryset: QuerySet, fields: Iterable[str], message: Optional[str] = ...): ...
    def set_context(self, serializer: BaseSerializer) -> None: ...
    def enforce_required_fields(self, attrs: Container[str]) -> None: ...
    def filter_queryset(self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def exclude_current_instance(self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any]) -> None: ...

class BaseUniqueForValidator(object):
    message: str = ...
    missing_message: str = ...

    queryset: QuerySet
    field: str
    date_field: str
    field_name: str
    date_field_name: str
    instance: Any
    def __init__(self, queryset: QuerySet, field: str, date_field: str, message: Optional[str] = ...): ...
    def set_context(self, serializer: BaseSerializer) -> None: ...
    def enforce_required_fields(self, attrs: Container[str]) -> None: ...
    def filter_queryset(self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def exclude_current_instance(self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T]) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any]) -> None: ...

class UniqueForDateValidator(BaseUniqueForValidator): ...
class UniqueForMonthValidator(BaseUniqueForValidator): ...
class UniqueForYearValidator(BaseUniqueForValidator): ...



from mypy.nodes import TypeInfo
from mypy.plugin import ClassDefContext, FunctionContext
from mypy.types import AnyType, Instance, Type, TypeOfAny

from mypy_drf_plugin import helpers


def retrieve_field_get_type(type_info: TypeInfo, private_field_name: str) -> Type:
    if not type_info.has_readable_member(private_field_name):
        return AnyType(TypeOfAny.explicit)

    sym = type_info.get(private_field_name)
    if sym and isinstance(sym.type, Instance):
        return sym.type

    return AnyType(TypeOfAny.explicit)


def set_types_metadata(ctx: ClassDefContext) -> None:
    actual_field_datatype = retrieve_field_get_type(ctx.cls.info,
                                                    private_field_name='_pyi_field_actual_type')
    primitive_field_datatype = retrieve_field_get_type(ctx.cls.info,
                                                       private_field_name='_pyi_field_primitive_type')

    # add types to metadata for use in ModelSerializer.Meta.fields
    metadata = helpers.get_drf_metadata(ctx.cls.info)
    if 'types' not in metadata:
        metadata['types'] = {'actual': actual_field_datatype.serialize(),
                             'primitive': primitive_field_datatype.serialize()}

    sym = ctx.api.lookup_fully_qualified_or_none(helpers.FIELD_FULLNAME)
    if sym and isinstance(sym.node, TypeInfo):
        helpers.get_drf_metadata(sym.node)['field_bases'][ctx.cls.fullname] = 1


def set_generic_parameters_for_field(ctx: FunctionContext) -> Type:
    default_return_type = ctx.default_return_type
    if not isinstance(default_return_type, Instance):
        return default_return_type

    types = helpers.get_drf_metadata_key(default_return_type.type, 'types', traverse_mro=True)
    if types is None:
        return default_return_type

    actual_field_datatype = helpers.deserialize_type(ctx.api, types['actual'])
    primitive_field_datatype = helpers.deserialize_type(ctx.api, types['primitive'])

    is_nullable = bool(helpers.parse_bool(helpers.get_argument_by_name(ctx, 'allow_null')))
    if is_nullable:
        actual_field_datatype = helpers.make_optional(actual_field_datatype)
        primitive_field_datatype = helpers.make_optional(primitive_field_datatype)

    return helpers.reparametrize_instance(default_return_type, [actual_field_datatype,
                                                                primitive_field_datatype])



from typing import Any, Optional, Tuple, Type

from django.db.models import Model
from django.contrib.auth import authenticate as authenticate
from django.middleware.csrf import CsrfViewMiddleware
from rest_framework.request import Request

def get_authorization_header(request: Request) -> bytes: ...

class CSRFCheck(CsrfViewMiddleware): ...

class BaseAuthentication(object):
    def authenticate(self, request: Request) -> Optional[Tuple[Any, Any]]: ...
    def authenticate_header(self, request: Request) -> Optional[str]: ...

class BasicAuthentication(BaseAuthentication):
    www_authenticate_realm: str = ...
    def authenticate_credentials(
        self, userid: str, password: str, request: Optional[Request] = ...
    ) -> Tuple[Any, None]: ...

class SessionAuthentication(BaseAuthentication):
    def enforce_csrf(self, request: Request) -> None: ...

class TokenAuthentication(BaseAuthentication):
    keyword: str = ...
    model: Optional[Type[Model]] = ...
    def get_model(self) -> Type[Model]: ...
    def authenticate_credentials(self, key: str) -> Tuple[Any, Any]: ...

class RemoteUserAuthentication(BaseAuthentication):
    header: str = ...



from typing import Any, Dict

from django.db.models import Model
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer

class CreateModelMixin(object):
    def create(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_create(self, serializer: BaseSerializer) -> None: ...
    def get_success_headers(self, data: Any) -> Dict[str, str]: ...

class ListModelMixin(object):
    def list(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class RetrieveModelMixin(object):
    def retrieve(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class UpdateModelMixin(object):
    def update(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_update(self, serializer: BaseSerializer) -> None: ...
    def partial_update(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

class DestroyModelMixin(object):
    def destroy(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...
    def perform_destroy(self, instance: Model) -> None: ...



from mypy.plugin import MethodContext
from mypy.types import Instance, Type

from mypy_drf_plugin import helpers


def return_typeddict_from_to_representation(ctx: MethodContext) -> Type:
    serializer_type = ctx.type
    if not isinstance(serializer_type, Instance):
        return ctx.default_return_type

    typeddict_type = helpers.get_corresponding_typeddict(serializer_type, ctx.api,
                                                         use_primitive_types=True)
    return typeddict_type


def return_list_of_typeddict_for_list_serializer_from_to_representation(ctx: MethodContext) -> Type:
    serializer_type = ctx.type
    if not isinstance(serializer_type, Instance):
        return ctx.default_return_type

    child_sym = serializer_type.type.get('child')
    if child_sym is None or not isinstance(child_sym.type, Instance):
        return ctx.default_return_type

    child_typeddict_type = helpers.get_corresponding_typeddict(child_sym.type, ctx.api,
                                                               use_primitive_types=True)
    return ctx.api.named_generic_type('builtins.list', [child_typeddict_type])


def return_typeddict_from_to_internal_value(ctx: MethodContext) -> Type:
    serializer_type = ctx.type
    if not isinstance(serializer_type, Instance):
        return ctx.default_return_type

    typeddict_type = helpers.get_corresponding_typeddict(serializer_type, ctx.api,
                                                         use_primitive_types=False)
    return typeddict_type


def return_list_of_typeddict_for_list_serializer_from_to_internal_value(ctx: MethodContext) -> Type:
    serializer_type = ctx.type
    if not isinstance(serializer_type, Instance):
        return ctx.default_return_type

    child_sym = serializer_type.type.get('child')
    if child_sym is None or not isinstance(child_sym.type, Instance):
        return ctx.default_return_type

    child_typeddict_type = helpers.get_corresponding_typeddict(child_sym.type, ctx.api,
                                                               use_primitive_types=False)
    return ctx.api.named_generic_type('builtins.list', [child_typeddict_type])



import collections
from typing import Any, Dict, List, Mapping, MutableMapping, Iterator

from rest_framework.exceptions import ErrorDetail
from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer

class ReturnDict(dict):
    serializer: BaseSerializer

class ReturnList(list):
    serializer: BaseSerializer

class BoundField(object):
    """
    A field object that also includes `.value` and `.error` properties.
    Returned when iterating over a serializer instance,
    providing an API similar to Django forms and form fields.
    """

    value: Any
    fields: Dict[str, Field]
    errors: List[ErrorDetail]
    def __init__(self, field: Field, value: Any, errors: List[ErrorDetail], prefix: str = ...): ...
    def as_form_field(self) -> BoundField: ...

class JSONBoundField(BoundField): ...

class NestedBoundField(BoundField, Mapping[str, Field]):
    def __iter__(self) -> Iterator[str]: ...
    def __getitem__(self, key: str) -> Field: ...
    def __len__(self) -> int: ...

class BindingDict(MutableMapping[str, Field]):
    serializer: BaseSerializer
    fields: Dict[str, Field]
    def __init__(self, serializer: BaseSerializer): ...
    def __setitem__(self, key: str, field: Field) -> None: ...
    def __getitem__(self, key: str) -> Field: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...



from typing import Any, Dict, List, Sequence, Type

from django.db.models import Model, QuerySet
from django.views import View
from rest_framework.request import Request

SAFE_METHODS: Sequence[str] = ("GET", "HEAD", "OPTIONS")

class OperationHolderMixin:
    def __and__(self, other: Any) -> OperandHolder: ...
    def __or__(self, other: Any) -> OperandHolder: ...
    def __rand__(self, other: Any) -> OperandHolder: ...
    def __ror__(self, other: Any) -> OperandHolder: ...

class OperandHolder(OperationHolderMixin):
    operator_class: Type[Any]
    op1_class: Type[Any]
    op2_class: Type[Any]
    def __init__(self, operator_class: Type[Any], op1_class: Type[Any], op2_class: Type[Any]): ...
    def __call__(self, *args, **kwargs) -> Any: ...

class AND:
    def __init__(self, op1: BasePermission, op2: BasePermission) -> None: ...
    def has_permission(self, request: Request, view: View) -> bool: ...
    def has_object_permission(self, request: Request, view: View, obj: Any) -> bool: ...

class OR:
    def __init__(self, op1: BasePermission, op2: BasePermission) -> None: ...
    def has_permission(self, request: Request, view: View) -> bool: ...
    def has_object_permission(self, request: Request, view: View, obj: Any) -> bool: ...

class BasePermissionMetaclass(OperationHolderMixin, type): ...

class BasePermission(object):
    def has_permission(self, request: Request, view: View) -> bool: ...
    def has_object_permission(self, request: Request, view: View, obj: Any) -> bool: ...

class AllowAny(BasePermission): ...
class IsAuthenticated(BasePermission): ...
class IsAdminUser(BasePermission): ...
class IsAuthenticatedOrReadOnly(BasePermission): ...

class DjangoModelPermissions(BasePermission):
    perms_map: Dict[str, List[str]] = ...

    authenticated_users_only: bool = ...
    def get_required_permissions(self, method: str, model_cls: Type[Model]) -> List[str]: ...
    def _queryset(self, view: View) -> QuerySet[Any]: ...

class DjangoModelPermissionsOrAnonReadOnly(DjangoModelPermissions): ...

class DjangoObjectPermissions(DjangoModelPermissions):
    def get_required_object_permissions(self, method: str, model_cls: Type[Model]) -> List[str]: ...



from typing import Any, Dict, Mapping, Optional, Sequence, Type

DEFAULTS: Dict[str, Any] = ...

IMPORT_STRINGS: Sequence[str] = ...

REMOVED_SETTINGS: Sequence[str] = ...

def perform_import(val: Optional[Any], setting_name: str) -> Optional[Any]: ...
def import_from_string(val: Optional[Any], setting_name: str) -> Type[Any]: ...

class APISettings(object):
    defaults: Mapping[str, Any]
    import_strings: Sequence[str]
    def __init__(
        self,
        user_settings: Optional[Mapping[str, Any]] = ...,
        defaults: Optional[Mapping[str, Any]] = ...,
        import_strings: Optional[Sequence[str]] = ...,
    ): ...
    @property
    def user_settings(self) -> Mapping[str, Any]: ...
    def __getattr__(self, attr: str) -> Any: ...
    def __check_user_settings(self, user_settings: Mapping[str, Any]) -> Mapping[str, Any]: ...
    def reload(self) -> None: ...

api_settings: APISettings = ...

def reload_api_settings(*args: Any, **kwargs: Any) -> None: ...



from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Protocol, Sequence, Union

from django.db.models import Model
from django.db.models.query import QuerySet
from rest_framework.fields import Field, _Validator
from rest_framework.request import Request

def method_overridden(method_name: str, klass: type, instance: Any) -> bool: ...

class ObjectValueError(ValueError): ...
class ObjectTypeError(TypeError): ...

class Hyperlink(str):
    is_hyperlink: bool = ...
    obj: Any
    def __new__(cls, url: str, obj: Any) -> Hyperlink: ...
    @property
    def name(self) -> str: ...

class PKOnlyObject(object):
    def __init__(self, pk: Any) -> None: ...

MANY_RELATION_KWARGS: Sequence[str] = ...

_Choices = Dict[Any, Any]

class Option(Protocol):
    start_option_group: bool = ...
    end_option_group: bool = ...
    label: str
    value: Any
    display_text: str

class RelatedField(Field):
    def __init__(
        self,
        many: bool = ...,
        allow_empty: bool = ...,
        queryset: Optional[QuerySet] = ...,
        html_cutoff: Optional[int] = ...,
        html_cutoff_text: str = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: Union[Callable, str] = ...,
        label: Optional[str] = ...,
        help_text: str = ...,
        allow_null: bool = ...,
        validators: Optional[Sequence[_Validator]] = ...,
        error_messages: Optional[Mapping[str, str]] = ...,
        style: Optional[Mapping[str, Any]] = ...,
    ): ...
    queryset: Optional[QuerySet] = ...
    html_cutoff: Optional[int] = ...
    html_cutoff_text: Optional[str] = ...
    @classmethod
    def many_init(cls, *args: Any, **kwargs: Any) -> ManyRelatedField: ...
    def get_queryset(self) -> QuerySet: ...
    def use_pk_only_optimization(self) -> bool: ...
    def get_attribute(self, instance: Any) -> Any: ...
    def get_choices(self, cutoff: Optional[int] = ...) -> _Choices: ...
    @property
    def choices(self) -> _Choices: ...
    @property
    def grouped_choices(self) -> _Choices: ...
    def iter_options(self) -> Iterable[Option]: ...
    def display_value(self, instance: Any) -> str: ...

class StringRelatedField(RelatedField): ...
class PrimaryKeyRelatedField(RelatedField): ...

class HyperlinkedRelatedField(RelatedField):
    lookup_field: str = ...
    lookup_url_kwarg: str = ...
    format: Optional[str] = ...
    view_name: Optional[str] = ...
    def __init__(self, view_name: Optional[str] = ..., **kwargs: Any): ...
    def get_object(self, view_name: str, view_args: Any, view_kwargs: Any) -> Model: ...
    def get_url(self, obj: Model, view_name: str, request: Request, format: str) -> str: ...

class HyperlinkedIdentityField(HyperlinkedRelatedField): ...

class SlugRelatedField(RelatedField):
    slug_field: Optional[str] = ...
    def __init__(self, slug_field: Optional[str] = None, **kwargs: Any): ...

class ManyRelatedField(Field):
    initial: List[object] = ...
    default_empty_html: List[object] = ...
    html_cutoff: Optional[int] = ...
    html_cutoff_text: Optional[str] = ...
    def __init__(self, child_relation: Optional[RelatedField] = ..., *args: Any, **kwargs: Any) -> None: ...
    def get_choices(self, cutoff: Optional[int] = ...) -> _Choices: ...
    @property
    def choices(self) -> _Choices: ...
    @property
    def grouped_choices(self) -> _Choices: ...
    def iter_options(self) -> Iterable[Option]: ...



from collections import OrderedDict
from typing import Any, Dict, Optional

from mypy.nodes import Expression, SymbolTableNode, TypeInfo
from mypy.plugin import CheckerPluginInterface, ClassDefContext
from mypy.types import AnyType, Instance, Type, TypeOfAny, TypedDictType, deserialize_map
from mypy_django_plugin import helpers as django_helpers
# noinspection PyUnresolvedReferences
from mypy_django_plugin.helpers import get_argument_by_name, get_argument_type_by_name, get_assigned_value_for_class, \
    get_model_fullname_from_string, get_nested_meta_node_for_current_class, has_any_of_bases, is_none_expr, is_optional, \
    iter_over_assignments, make_optional, make_required, parse_bool, reparametrize_instance


FIELD_FULLNAME = 'rest_framework.fields.Field'
BASE_SERIALIZER_FULLNAME = 'rest_framework.serializers.BaseSerializer'
SERIALIZER_FULLNAME = 'rest_framework.serializers.Serializer'
LIST_SERIALIZER_FULLNAME = 'rest_framework.serializers.ListSerializer'
MODEL_SERIALIZER_FULLNAME = 'rest_framework.serializers.ModelSerializer'

# TODO: finish mapping
SERIALIZER_FIELD_MAPPING = {
    'django.db.models.fields.AutoField': 'rest_framework.serializers.IntegerField',
    'django.db.models.fields.BigIntegerField': 'rest_framework.serializers.IntegerField',
    'django.db.models.fields.BooleanField': 'rest_framework.serializers.BooleanField',
    'django.db.models.fields.CharField': 'rest_framework.serializers.CharField',
    'django.db.models.fields.CommaSeparatedIntegerField': 'rest_framework.serializers.CharField',
    'django.db.models.fields.DateField': 'rest_framework.serializers.DateField',
    'django.db.models.fields.DateTimeField': 'rest_framework.serializers.DateTimeField',
    'django.db.models.fields.DecimalField': 'rest_framework.serializers.DecimalField',
}
ID_TYPE = 'builtins.object'


def get_field_type(typ: Instance, primitive: bool = False) -> Type:
    if primitive:
        return typ.args[1]
    else:
        return typ.args[0]


def deserialize_type(api: CheckerPluginInterface, data: Dict[str, Any]) -> Type:
    if isinstance(data, str):
        return api.named_generic_type(data, [])

    classname = data['.class']
    deserialize = deserialize_map.get(classname)
    if not deserialize:
        return AnyType(TypeOfAny.from_error)
    return deserialize(data)


def get_type_for_model_field(model_field_type: Instance,
                             api: CheckerPluginInterface,
                             use_primitive_types: bool = False) -> Type:
    field_type_fullname = SERIALIZER_FIELD_MAPPING.get(model_field_type.type.fullname())
    if not field_type_fullname:
        return AnyType(TypeOfAny.explicit)

    field_type = api.named_generic_type(field_type_fullname, [])
    types = get_drf_metadata_key(field_type.type, 'types', traverse_mro=True)
    if types:
        if use_primitive_types:
            typ_data = types['primitive']
        else:
            typ_data = types['actual']
        typ = deserialize_type(api, typ_data)
        return typ

    return AnyType(TypeOfAny.explicit)


def is_autogenerated_primary_key(sym: SymbolTableNode) -> bool:
    return sym and isinstance(sym.type, Instance) and sym.type.type.fullname() == ID_TYPE


def get_corresponding_typeddict(serializer_type: Instance,
                                api: CheckerPluginInterface,
                                use_primitive_types: bool = False) -> Type:
    typeddict_items = OrderedDict()  # type: OrderedDict[str, Type]
    for base in serializer_type.type.mro:
        for name, sym in base.names.items():
            if name in typeddict_items:
                continue
            typ = sym.type
            if isinstance(typ, Instance) and typ.type.has_base(FIELD_FULLNAME):
                typeddict_items[name] = get_field_type(typ, primitive=use_primitive_types)

    if serializer_type.type.has_base(MODEL_SERIALIZER_FULLNAME):
        base_model_fullname = get_drf_metadata(serializer_type.type).get('base_model')
        if base_model_fullname is not None:
            try:
                base_model_type = api.named_generic_type(base_model_fullname, []).type
            except (AssertionError, KeyError):
                # TODO: remove and figure out circular imports after semanal refactoring
                # not yet imported
                return AnyType(TypeOfAny.from_unimported_type)

            defined_model_fields = get_drf_metadata(serializer_type.type).get('fields', [])
            for field_name in defined_model_fields:
                if field_name not in typeddict_items:
                    sym = base_model_type.get(field_name)
                    if field_name == 'id' and is_autogenerated_primary_key(sym):
                        typeddict_items[field_name] = api.named_generic_type('builtins.int', [])
                        continue

                    if sym and isinstance(sym.type, Instance) and sym.type.type.has_base(django_helpers.FIELD_FULLNAME):
                        typeddict_items[field_name] = get_type_for_model_field(sym.type, api=api)
                        continue

                    typeddict_items[field_name] = AnyType(TypeOfAny.from_unimported_type)

    return TypedDictType(items=typeddict_items,
                         required_keys=set(typeddict_items.keys()),
                         fallback=api.named_generic_type('builtins.dict', [api.named_generic_type('builtins.str', []),
                                                                           AnyType(TypeOfAny.explicit)]))


def get_drf_metadata(info: TypeInfo) -> Dict[str, Any]:
    return info.metadata.setdefault('drf', {})


def get_drf_metadata_key(info: TypeInfo,
                         key: str,
                         traverse_mro: bool = False) -> Optional[Any]:
    infos_to_check = [info]
    if traverse_mro:
        infos_to_check = info.mro

    for base in infos_to_check:
        metadata = get_drf_metadata(base)
        if key in metadata:
            return metadata[key]

    return None


def get_meta_attribute_value(ctx: ClassDefContext, attr_name: str) -> Optional[Expression]:
    meta_info = get_nested_meta_node_for_current_class(ctx.cls.info)
    if meta_info is None:
        return None

    return get_assigned_value_for_class(meta_info, attr_name)



from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers

class AuthTokenSerializer(serializers.Serializer):
    username = serializers.CharField(label=_("Username"))
    password = serializers.CharField(label=_("Password"), style={"input_type": "password"}, trim_whitespace=False)



from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

class Token(models.Model):
    key = models.CharField(_("Key"), max_length=40, primary_key=True)
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL, related_name="auth_token", on_delete=models.CASCADE, verbose_name=_("User")
    )
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    def generate_key(self) -> str: ...
